#include "qwatermark.h"
#include "QDebug"

QWatermark::QWatermark(QWidget *parent)
    : QMainWindow(parent)
{
	ui.setupUi(this);

	colorDialog = new QColorDialog();

	//Signal&Slot connection
	QObject::connect(ui.sourcePushButton,SIGNAL(clicked()),this,SLOT(selectSourceFolder(void)));
	QObject::connect(ui.destinationPushButton,SIGNAL(clicked()),this,SLOT(selectDestinationFolder(void)));
	QObject::connect(ui.imageRadioButton,SIGNAL(toggled(bool)),this,SLOT(imageTextChange(void)));
	QObject::connect(ui.textRadioButton,SIGNAL(toggled(bool)),this,SLOT(imageTextChange(void)));
	QObject::connect(ui.startPushButton,SIGNAL(clicked()),this,SLOT(doWatermark(void)));
	QObject::connect(ui.watermarkPushButton,SIGNAL(clicked()),this,SLOT(selectLogo(void)));
	QObject::connect(ui.colorBoxPushButton,SIGNAL(clicked()),this,SLOT(showColorDialog(void)));
	QObject::connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(about(void)));
	QObject::connect(ui.transparencyHorizontalSlider, SIGNAL(valueChanged(int)), ui.transparencySpinBox, SLOT(setValue(int)));
	QObject::connect(ui.transparencySpinBox, SIGNAL(valueChanged(int)), ui.transparencyHorizontalSlider, SLOT(setValue(int)));

	//Validator for number only
	QValidator *validator = new QIntValidator(0, 9999, this);
	ui.widthLineEdit->setValidator(validator);
	ui.heightLineEdit->setValidator(validator);
}

//Select source folder for images
void QWatermark::selectSourceFolder(void)
{
	//Create a dialog to select a folder
	QString sourceDirectoryName = QFileDialog::getExistingDirectory(this,tr("Open Source Directory"),
			"",QFileDialog::ShowDirsOnly);

	//Assign of the selected folder to the line edit
	ui.sourceLineEdit->setText(sourceDirectoryName);

	//Create a QDir to access the content of the folder passed a parameter
	QDir sourceDir = QDir(sourceDirectoryName);

	//I need only files into the selected directory (no other folder or link for example)
	sourceDir.setFilter(QDir::Files);

	//And I need only some file types
	QStringList filter;
	filter << "*.bmp" << "*.gif" << "*.jpg" << "*.jpeg" << "*.png" << "*.pbm" << "*.pgm" << "*.ppm"
			<< "*.tiff" << "*.xbm" << "*.xpm";
	sourceDir.setNameFilters(filter);

	//Create a list of all file
	filesToProcess = new QStringList(sourceDir.entryList());

	//reset ProgressBar
	ui.totalProgressBar->reset();
}

//Select destination folder for images (similar to source)
void QWatermark::selectDestinationFolder(void)
{
	QString destinationDirectoryName = QFileDialog::getExistingDirectory(this,tr("Open Destination Directory"),
			"",QFileDialog::ShowDirsOnly);
	ui.destinationLineEdit->setText(destinationDirectoryName);
}

//Select the image to use to watermark
void QWatermark::selectLogo(void)
{
	QString logoFile = QFileDialog::getOpenFileName(this, tr("Select File"),"",
			tr("Images (*. bmp *.gif *.jpg *.jpeg *.png *.pbm *.pgm *.ppm *.tiff *.xbm *.xpm)"));
	ui.watermarkLineEdit->setText(logoFile);

}

//Switch between image and text watermarking
void QWatermark::imageTextChange(void)
{
	if(ui.textRadioButton->isChecked()){
		//Disable image part
		ui.selectImageForLabel->setDisabled(true);
		ui.watermarkLineEdit->setDisabled(true);
		ui.watermarkPushButton->setDisabled(true);
		//and enable text part
		ui.textLabel->setEnabled(true);
		ui.textLineEdit->setEnabled(true);
		ui.fontLabel->setEnabled(true);
		ui.fontComboBox->setEnabled(true);
		ui.colorBoxPushButton->setEnabled(true);
		ui.boldToolButton->setEnabled(true);
		ui.italicToolButton->setEnabled(true);
		ui.underlineToolButton->setEnabled(true);
		ui.sizeTextComboBox->setEnabled(true);
		ui.sizeTextLabel->setEnabled(true);
		//The contrary
	}else if(ui.imageRadioButton->isChecked()){
		ui.selectImageForLabel->setDisabled(false);
		ui.watermarkLineEdit->setDisabled(false);
		ui.watermarkPushButton->setDisabled(false);
		ui.textLabel->setEnabled(false);
		ui.textLineEdit->setEnabled(false);
		ui.fontLabel->setEnabled(false);
		ui.fontComboBox->setEnabled(false);
		ui.colorBoxPushButton->setEnabled(false);
		ui.boldToolButton->setEnabled(false);
		ui.italicToolButton->setEnabled(false);
		ui.underlineToolButton->setEnabled(false);
		ui.sizeTextComboBox->setEnabled(false);
		ui.sizeTextLabel->setEnabled(false);
	}

}


void QWatermark::showColorDialog(void)
{
	selectedColor = colorDialog->getColor();
}

//Execute the watermark
void QWatermark::doWatermark(void)
{
	QImage resultImage;
	QSize imageSize;
	QImage logo;
	QString watermarkString;
	QPainter painter;
	QString newExtension = ui.outputComboBox->currentText();
	int imageQuality = ui.outputSpinBox->value();
	int imageX = 0;
	int imageY = 0;

	//Check if source path is written
	if(ui.sourceLineEdit->text().isEmpty()){
		//Show error box
		QMessageBox::information(this, tr("Empty Path"),
						tr("Please enter a path to load images."));
		return;
	}

	//Check if destination path is written
	if(ui.destinationLineEdit->text().isEmpty()){
		//Show error box
		QMessageBox::information(this, tr("Empty Path"),
						tr("Please enter a path to save images."));
		return;
	}

	//Check if resize option is checked
	if(ui.resizeGroupBox->isChecked()){
		//set width and height
		if(!ui.widthLineEdit->text().isEmpty())
			imageSize.setWidth(ui.widthLineEdit->text().toInt());
		if(!ui.heightLineEdit->text().isEmpty())
			imageSize.setHeight(ui.heightLineEdit->text().toInt());
	}

	//Check if you chose to use an image  or a text for the watermark
	if(ui.imageRadioButton->isChecked()){
		//Check if the image path is written
		if(ui.watermarkLineEdit->text().isEmpty()){
			//Show error box
			QMessageBox::information(this, tr("Empty Path"),
							tr("Please enter a path for the watermark image."));
			return;
		}else{
			//Load the image
			logo.load(ui.watermarkLineEdit->text());
		}

	}else if(ui.textRadioButton->isChecked()){
		watermarkString = ui.textLineEdit->text();
	}

	//Set ProgressBar
	ui.totalProgressBar->setMinimum(1);
	ui.totalProgressBar->setMaximum(filesToProcess->count());

	//Iterate over images
	for(int i = 0; i < filesToProcess->size(); ++i){

		//Load image
		resultImage.load(ui.sourceLineEdit->text() + "/" + filesToProcess->at(i)); //Da verificare se funziona!
		if(painter.begin(&resultImage)){
			painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
			//Check the transparency value
			painter.setOpacity((float)(ui.transparencySpinBox->value())/100);

			//Draw image
			if(ui.imageRadioButton->isChecked()){
				//controls which logo position is selected
				if(ui.ULRadioButton->isChecked()){
					//Do nothing, is default already set
				}
				if(ui.UCRadioButton->isChecked()){
					imageX = resultImage.width()/2 - logo.width()/2;
					imageY = 0;
				}
				if(ui.URRadioButton->isChecked()){
					imageX = resultImage.width() - logo.width();
					imageY = 0;
				}
				if(ui.CLRadioButton->isChecked()){
					imageX = 0;
					imageY = resultImage.height()/2 - logo.height()/2;
				}
				if(ui.CCRadioButton->isChecked()){
					imageX = resultImage.width()/2 - logo.width()/2;
					imageY = resultImage.height()/2 - logo.height()/2;
				}
				if(ui.CRRadioButton->isChecked()){
					imageX = resultImage.width() - logo.width();
					imageY = resultImage.height()/2 - logo.height()/2;
				}
				if(ui.LLRadioButton->isChecked()){
					imageX = 0;
					imageY = resultImage.height() - logo.height();
				}
				if(ui.LCRadioButton->isChecked()){
					imageX = resultImage.width()/2 - logo.width()/2;
					imageY = resultImage.height() - logo.height();
				}
				if(ui.LRRadioButton->isChecked()){
					imageX = resultImage.width() - logo.width();
					imageY = resultImage.height() - logo.height();
				}

				//set the position
				painter.drawImage(imageX,imageY,logo,0,0,-1,-1);

				painter.end();

			}else if(ui.textRadioButton->isChecked()){ //Draw text
				//Check the transparency value
					painter.setOpacity((float)(ui.transparencySpinBox->value())/100);
				//set font proprierty
				QFont font = ui.fontComboBox->currentFont();
				if(ui.boldToolButton->isChecked())
					font.setBold(true);
				if(ui.italicToolButton->isChecked())
					font.setItalic(true);
				if(ui.underlineToolButton->isChecked())
					font.setUnderline(true);
				font.setPointSizeF(ui.sizeTextComboBox->currentText().toFloat());
				painter.setFont(font);
				//set font color
				painter.setPen(selectedColor);
				QRect rect(0, 0, resultImage.width(), resultImage.height());
				//draw text in the correct position
				if(ui.ULRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignLeft | Qt::AlignTop, watermarkString);
				if(ui.UCRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignHCenter | Qt::AlignTop, watermarkString);
				if(ui.URRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignRight | Qt::AlignTop, watermarkString);
				if(ui.CLRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, watermarkString);
				if(ui.CCRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignCenter, watermarkString);
				if(ui.CRRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignRight | Qt::AlignVCenter, watermarkString);
				if(ui.LLRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignLeft | Qt::AlignBottom, watermarkString);
				if(ui.LCRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignHCenter | Qt::AlignBottom, watermarkString);
				if(ui.LRRadioButton->isChecked())
					painter.drawText(rect, Qt::AlignRight | Qt::AlignBottom, watermarkString);

				painter.end();
			}

			//Scale the image if request
			if(ui.resizeGroupBox->isChecked()){
				if(ui.aspectRationCheckBox->isChecked()){
					if(ui.heightLineEdit->text().isEmpty())
						 resultImage = resultImage.scaledToWidth(imageSize.width(), Qt::SmoothTransformation);
						else if(ui.widthLineEdit->text().isEmpty())
							resultImage = resultImage.scaledToHeight(imageSize.height(), Qt::SmoothTransformation);
						else
							resultImage = resultImage.scaled(imageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
				}else{
					if(!ui.widthLineEdit->text().isEmpty() && !ui.heightLineEdit->text().isEmpty())
						resultImage = resultImage.scaled(imageSize,Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
					else{
						QMessageBox::information(this, tr("Empty Field"),
								tr("Set a width and a height for the output images."));
						return;
					}
				}
			}

			//save the image
			if(!resultImage.save(ui.destinationLineEdit->text() + "/" //Da verificare se funziona!
					+ filesToProcess->at(i).section('.',0,-2) + newExtension, 0, imageQuality)){
				QMessageBox::warning(this, tr("Error"),
						tr("An error occurred while saving the image '%1'").arg(filesToProcess->at(i).section('.',0,-2) + newExtension));
			}
			ui.totalProgressBar->setValue(i+1);
		}
	}
	QMessageBox::information(this, tr("Success"), tr("Processing Completed."));

}

void QWatermark::about(void)
 {
     QMessageBox::about(this, tr("About QWatermark"),
                        tr("<b>QWatermark:</b> a simple program to "
                           "impress text or logo over images."));
 }

QWatermark::~QWatermark()
{

}
